"""functions to generate api"""

import functools

from cp_core.libs.total import controller
from cp_core.libs.total.basic import BasicFuncSet


def get_compute(funcset: BasicFuncSet):
    """隐藏了其他函数的复杂性，直接调用本函数即可完成 compute 函数的生成"""
    return functools.partial(
        controller.compute,
        collect_files=functools.partial(
            controller.collect_files,
            is_write=False,
            compute_single=functools.partial(
                controller.compute_single,
                funcset=funcset,
            ),
        ),
    )


def get_steps(fn: BasicFuncSet):
    """获取函数集的每一个步骤，可以直接生成新的 api"""

    from cp_core.libs import proto
    from cp_core.libs.core.filter.controller import filter_controller
    from cp_core.libs.core.model.controller import model_controller
    from cp_core.libs.core.stat.controller import stat_controller

    use_filter: proto.UseFilter = functools.partial(
        filter_controller, process_data=fn.process_data
    )
    use_stat: proto.UseStat = functools.partial(stat_controller, process_data=fn.stat)
    use_model: proto.UseModel = functools.partial(
        model_controller, process_data=fn.model
    )
    return use_filter, use_stat, use_model
